@@ -397,7 +397,7 @@ impl Display for ContextError {
         match self {
             #[cfg(feature = "defensive")]
             Self::ModuleNotSane => f.write_str("Module not sane!"),
-            Self::MemoryAllocFailed => f.write_str("Memory allocation failed"),
+            Self::MemoryAllocFailed => f.write_str("Memory allocation failed!"),
             Self::Unknown(number) => f.write_fmt(format_args!("Unknown error: {}!", number)),
         }
     }
